package lib;

import lib.Vec.Vec3;

/**
	Reporesents a Point in a 3D `Float` System.
	Basicly a wrapper for Vec3<Float> with some extra functions.
	`Y` represents the height of the point.
**/
@:forward
abstract WorldPos(Vec3<Float>) from Vec3<Float> to Vec3<Float> {
	public inline function new(v:Vec3<Float>) {
		this = v;
	}

	@:op(A + B)
	public inline function add(rhs:WorldPos):WorldPos {
		return this.add(rhs);
	}

	@:op(A - B)
	public inline function sub(rhs:Vec3<Float>):WorldPos {
		return this.sub(rhs);
	}

	@:op(A * B)
	public inline function multiplyScalar(rhs:Float):WorldPos {
		return this.multiplyScalar(rhs);
	}

	@:op(A / B)
	public function divideScalar(rhs:Float):WorldPos {
		return {
			y: this.y / rhs,
			x: this.x / rhs,
			z: this.z / rhs
		};
	}

	@:op(-A)
	public function negate():WorldPos {
		return this.negate();
	}

	public function normalize():WorldPos {
		var l = this.length();
		return multiplyScalar(1 / l);
	}

	@:op(A == B)
	public function equals(rhs:WorldPos):Bool {
		return close(rhs);
	}

	@:op(A != B)
	public function notEquals(rhs:WorldPos):Bool {
		return !close(rhs);
	}

	public function close(rhs:WorldPos, epsilon:Float = 0.001):Bool {
		return Math.abs(this.x - rhs.x) < epsilon && Math.abs(this.y - rhs.y) < epsilon && Math.abs(this.z - rhs.z) < epsilon;
	}

	public function toString():String {
		return 'WorldPos(${this.x}, ${this.y}, ${this.z})';
	}

	public function round():WorldPos {
		return new WorldPos({
			x: Math.fround(this.x),
			y: Math.fround(this.y),
			z: Math.fround(this.z)
		});
	}
}
